/*
 * COPY OF EXPORT TO THE VIEWER 
 *
 */
 
#include <Pilot.h>

#include <SerialMgr.h>

typedef struct {
	UInt refNum;
	SerSettingsType serSettings;
} libg;

Err OpenSerial(libg *g)
{
    DWord err = 0;
    
    err = SysLibFind("Serial Library", &g->refNum);
    if (err) { 
		g->refNum = -1;
    	return err; 
    }
    err = SerOpen(g->refNum, 0, 19200);
    if (err) { 
    	return err; 
    }
    err = SerGetSettings(g->refNum, &g->serSettings);
    if (err) { 
    	return err; 
    }
    g->serSettings.flags = serSettingsFlagBitsPerChar8 | serSettingsFlagStopBits1;
    err = SerSetSettings(g->refNum, &g->serSettings);
    if (err) { 
    	return err; 
    }
    return err;
}


Err CloseSerial(libg *g)
{
	SysTaskDelay(SysTicksPerSecond());
	if (g->refNum != -1)
   	SerClose(g->refNum);
   return 0;
}


void SendData(libg *g, char *buf) {
	Err err;
	int l;
	l=StrLen(buf);
	SerSend(g->refNum,buf,l,&err);
	SerSendWait(g->refNum,-1);
// add cr/lf
	SerSend(g->refNum,"\012\015",2,&err);
	SerSendWait(g->refNum,-1);
}

void SendHex(libg *g, unsigned char v) {
	Err err;
	char *d = "0123456789ABCDEF";
	SerSend(g->refNum,"0x",2,&err);
	SerSendWait(g->refNum,-1);
	SerSend(g->refNum,&d[((v>>4)&0xf)],1,&err);
	SerSendWait(g->refNum,-1);
	SerSend(g->refNum,&d[(v&0xf)],1,&err);
	SerSendWait(g->refNum,-1);
	SerSend(g->refNum,"\012\015",2,&err);
	SerSendWait(g->refNum,-1);
}

void SendRawData(libg *g, char *buf, short l) {
	while (l--)
		SendHex(g,*buf++);
}

void SendString(libg *g, char *str) {
	SendRawData(g,str,StrLen(str));
}

void Float2Str(float f, char *num) {
	char *p=num;
	long l;
	short d,nz=0;
	long x;
	x = 100000*f;
	if (x<0) {
		*p++='-';
		x=-x;
	}
	l=1000000000;
	while (l) {
		if (l==10000)
			*p++ = '.';
		d = x/l;
		if (d || nz || l<100000) {
			*p++ = '0'+d;
			nz = 1;
		}
		x = x%l;
		l/=10;
	}
	*p=0;
}

void SendFloat(libg *g, float f) {
	char num[32];
	char *p=num;
	long l;
	short d,nz=0;
	long x;
	x = 100000*f;
	if (x<0) {
		*p++='-';
		x=-x;
	}
	l=1000000000;
	while (l) {
		if (l==10000)
			*p++ = '.';
		d = x/l;
		if (d || nz || l<100000) {
			*p++ = '0'+d;
			nz = 1;
		}
		x = x%l;
		l/=10;
	}
	*p=0;
	SendData(g,num);
	SysTaskDelay(2);
}

void SendInt(libg *g, Int i) {
	char num[32];
	StrIToA(num,i);
	SendData(g,num);
	SysTaskDelay(2);
}

typedef int fix_t;
typedef fix_t vector_t[4];

typedef struct _s_stripEntry {
	short num_indexes;
	short max_indexes;
	Boolean hidden;
	VoidHand next;
	short indexes[1];
} _stripEntry, *_stripEntryP;
typedef VoidHand _stripEntryH;

typedef struct {
	short num_vertexes;
	short max_vertexes;
	_stripEntryH strips;
	vector_t vertexes[1];	
} _modelEntry,*_modelEntryP;
typedef VoidHand _modelEntryH;

/*
 *		ENTRY POINT
 */
void export(char *name,
				_modelEntryH model, 
				float (*convert)(fix_t v), 
				void (*progressing)(short p)) {
	int i,j=0;
	_modelEntryP mep;
	_stripEntryH seh,tseh;
	_stripEntryP sep;
	short tot;
	libg *g;
	char buf[64];

	g = (libg*)MemPtrNew(sizeof(libg));

	if (!g || OpenSerial(g)) {
		if (g) MemPtrFree(g);
		return;
	}

	mep = MemHandleLock(model);
	seh = mep->strips;
	tot = mep->num_vertexes;
	MemHandleUnlock(model);
	while (seh) {
		sep = MemHandleLock(seh);
		tseh = sep->next;
		tot += sep->num_indexes;
		MemHandleUnlock(seh);
		seh = tseh;
	}

	SendData(g,"SAVE");

	StrCopy(buf,name);
	StrCat(buf,".wrl");

	SendData(g,buf);

	SendString(g,"#VRML V2.0 utf8\r\n");
	SendString(g,"NavigationInfo { headlight TRUE }\r\n");
	SendString(g,"DEF m Material {\r\n");
	SendString(g,"ambientIntensity 0.5\r\n");
	SendString(g,"diffuseColor 1 0.5 0\r\n");
	SendString(g,"shininess 0.5\r\n");
	SendString(g,"specularColor 0 0 0.5 }\r\n");
	SendString(g,"Shape	{\r\n");
	SendString(g,"appearance Appearance { material USE m }\r\n");
	SendString(g,"geometry IndexedFaceSet {\r\n");
	SendString(g,"coord Coordinate {\r\n");
	SendString(g,"point [\r\n");

	mep = MemHandleLock(model);
	seh = mep->strips;
	for (i=0;i<mep->num_vertexes;i++) {

		Float2Str(convert(mep->vertexes[i][0]),buf);
		StrCat(buf," ");
		SendString(g,buf);
		Float2Str(convert(mep->vertexes[i][1]),buf);
		StrCat(buf," ");
		SendString(g,buf);
		Float2Str(convert(mep->vertexes[i][2]),buf);
		StrCat(buf,"\r\n");
		SendString(g,buf);

		progressing((j++)*100/tot);
	}
	MemHandleUnlock(model);

	SendString(g,"]\r\n");
	SendString(g,"}\r\n");
	SendString(g,"coordIndex [\r\n");

	while (seh) {
		sep = MemHandleLock(seh);
		tseh = sep->next;
		j+=2;
		for (i=2;i<sep->num_indexes;i++) {
			StrIToA(buf,sep->indexes[i-2]);
			StrCat(buf," ");
			SendString(g,buf);
			StrIToA(buf,sep->indexes[i-1]);
			StrCat(buf," ");
			SendString(g,buf);
			StrIToA(buf,sep->indexes[i]);
			StrCat(buf," -1\r\n");
			SendString(g,buf);
			progressing((j++)*100/tot);
		}
		MemHandleUnlock(seh);
		seh = tseh;
	}

	SendString(g,"]\r\n");
	SendString(g,"solid FALSE\r\n");
	SendString(g,"}\r\n");
	SendString(g,"}\r\n");

	SendData(g,"END");

	CloseSerial(g);
	MemPtrFree(g);
}
