/*
 * COPY OF EXPORT TO THE VIEWER 
 *
 */
 
#include <Pilot.h>

#include <SerialMgr.h>

typedef struct {
	UInt refNum;
	SerSettingsType serSettings;
} libg;

Err OpenSerial(libg *g)
{
    DWord err = 0;
    
    err = SysLibFind("Serial Library", &g->refNum);
    if (err) { 
		g->refNum = -1;
    	return err; 
    }
    err = SerOpen(g->refNum, 0, 19200);
    if (err) { 
    	return err; 
    }
    err = SerGetSettings(g->refNum, &g->serSettings);
    if (err) { 
    	return err; 
    }
    g->serSettings.flags = serSettingsFlagBitsPerChar8 | serSettingsFlagStopBits1;
    err = SerSetSettings(g->refNum, &g->serSettings);
    if (err) { 
    	return err; 
    }
    return err;
}


Err CloseSerial(libg *g)
{
	SysTaskDelay(SysTicksPerSecond());
	if (g->refNum != -1)
   	SerClose(g->refNum);
   return 0;
}


void SendData(libg *g, char *buf) {
	Err err;
	int l;
	l=StrLen(buf);
	SerSend(g->refNum,buf,l,&err);
	SerSendWait(g->refNum,-1);
// add cr/lf
	SerSend(g->refNum,"\012\015",2,&err);
	SerSendWait(g->refNum,-1);
}

void SendHex(libg *g, unsigned char v) {
	Err err;
	char *d = "0123456789ABCDEF";
	SerSend(g->refNum,"0x",2,&err);
	SerSendWait(g->refNum,-1);
	SerSend(g->refNum,&d[((v>>4)&0xf)],1,&err);
	SerSendWait(g->refNum,-1);
	SerSend(g->refNum,&d[(v&0xf)],1,&err);
	SerSendWait(g->refNum,-1);
	SerSend(g->refNum,"\012\015",2,&err);
	SerSendWait(g->refNum,-1);
}

void SendRawData(libg *g, char *buf, short l) {
	while (l--)
		SendHex(g,*buf++);
}

void SendFloat(libg *g, float f) {
	char num[32];
	char *p=num;
	long l;
	short d,nz=0;
	long x;
	x = 100000*f;
	if (x<0) {
		*p++='-';
		x=-x;
	}
	l=1000000000;
	while (l) {
		if (l==10000)
			*p++ = '.';
		d = x/l;
		if (d || nz || l<100000) {
			*p++ = '0'+d;
			nz = 1;
		}
		x = x%l;
		l/=10;
	}
	*p=0;
	SendData(g,num);
	SysTaskDelay(2);
}

void SendInt(libg *g, Int i) {
	char num[32];
	StrIToA(num,i);
	SendData(g,num);
	SysTaskDelay(2);
}

typedef int fix_t;
typedef fix_t vector_t[4];

typedef struct _s_stripEntry {
	short num_indexes;
	short max_indexes;
	Boolean hidden;
	VoidHand next;
	short indexes[1];
} _stripEntry, *_stripEntryP;
typedef VoidHand _stripEntryH;

typedef struct {
	short num_vertexes;
	short max_vertexes;
	_stripEntryH strips;
	vector_t vertexes[1];	
} _modelEntry,*_modelEntryP;
typedef VoidHand _modelEntryH;

/*
 *		ENTRY POINT
 */
void export(char *name,
				_modelEntryH model, 
				float (*convert)(fix_t v), 
				void (*progressing)(short p)) {
	libg *g;

	g = (libg*)MemPtrNew(sizeof(libg));

	if (!g || OpenSerial(g)) {
		if (g) MemPtrFree(g);
		return;
	}

	SendData(g,"SAVE");

	SendData(g,"filename.txt");

	progressing(33);
	SendRawData(g,"This is a testfile\r\n",20);
	progressing(66);
	SendRawData(g,"Generated from the export2 example\r\n",36);
	progressing(99);

	SendData(g,"END");

	CloseSerial(g);
	MemPtrFree(g);
}
